﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using Infragistics.Windows.DockManager;
using System.Windows;
using System.Windows.Controls;
using System.Threading;
using System.Windows.Threading;

namespace Infragistics.Extensions
{
	public static class ContentPaneExtensions
	{
		#region Member Variables

		private static readonly DependencyProperty DefaultStyleKeyProperty; 

		#endregion //Member Variables

		#region Constructor
		static ContentPaneExtensions()
		{
			// the FrameworkElement.DefaultStyleKeyProperty is protected
			DefaultStyleKeyProperty = typeof(FrameworkElement).GetField("DefaultStyleKeyProperty", System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.NonPublic).GetValue(null) as DependencyProperty;

			CommandManager.RegisterClassCommandBinding(typeof(ContentPane), new CommandBinding(FloatCommand, new ExecutedRoutedEventHandler(OnExecuteDockFloatCommand), new CanExecuteRoutedEventHandler(OnCanExecuteDockFloatCommand)));
			CommandManager.RegisterClassCommandBinding(typeof(ContentPane), new CommandBinding(DockCommand, new ExecutedRoutedEventHandler(OnExecuteDockFloatCommand), new CanExecuteRoutedEventHandler(OnCanExecuteDockFloatCommand)));
		} 
		#endregion //Constructor

		#region Commands
		/// <summary>
		/// Changes the PaneLocation of a ContentPane to Floating
		/// </summary>
		public static readonly RoutedCommand FloatCommand = new RoutedCommand("Float", typeof(ContentPaneExtensions));

		/// <summary>
		/// Changes the PaneLocation of a ContentPane to one of the Docked dockable locations
		/// </summary>
		public static readonly RoutedCommand DockCommand = new RoutedCommand("Dock", typeof(ContentPaneExtensions));

		#endregion //Commands

		#region Properties

		#region AddDockFloatOptions

		/// <summary>
		/// AddDockFloatOptions Attached Dependency Property
		/// </summary>
		public static readonly DependencyProperty AddDockFloatOptionsProperty =
			DependencyProperty.RegisterAttached("AddDockFloatOptions", typeof(bool), typeof(ContentPaneExtensions),
				new FrameworkPropertyMetadata((bool)false,
					new PropertyChangedCallback(OnAddDockFloatOptionsChanged)));

		/// <summary>
		/// Gets the AddDockFloatOptions property.  This dependency property determines if the options menu of a ContentPane should include "Dock" and "Float" options.
		/// </summary>
		public static bool GetAddDockFloatOptions(DependencyObject d)
		{
			return (bool)d.GetValue(AddDockFloatOptionsProperty);
		}

		/// <summary>
		/// Sets the AddDockFloatOptions property.  This dependency property determines if the options menu of a ContentPane should include "Dock" and "Float" options.
		/// </summary>
		public static void SetAddDockFloatOptions(DependencyObject d, bool value)
		{
			d.SetValue(AddDockFloatOptionsProperty, value);
		}

		private static void OnAddDockFloatOptionsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			ContentPane cp = d as ContentPane;

			if (cp == null)
				return;

			if (true.Equals(e.NewValue))
				cp.OptionsMenuOpening += new EventHandler<Windows.DockManager.Events.PaneOptionsMenuOpeningEventArgs>(OnDockFloatOptionsMenuOpening);
			else
				cp.OptionsMenuOpening -= new EventHandler<Windows.DockManager.Events.PaneOptionsMenuOpeningEventArgs>(OnDockFloatOptionsMenuOpening);
		}

		private static void OnDockFloatOptionsMenuOpening(object sender, Windows.DockManager.Events.PaneOptionsMenuOpeningEventArgs e)
		{
			ContentPane cp = sender as ContentPane;
			e.Items.Insert(0, CreateMenuItem(cp, FloatCommand, "Float"));
			e.Items.Insert(1, CreateMenuItem(cp, DockCommand, "Dock"));
			e.Items.Insert(2, CreateMenuItemSeparator());
		}
		#endregion // AddDockFloatOptions

		#endregion //Properties

		#region Methods

		#region CreateMenuItem
		private static MenuItem CreateMenuItem(ContentPane pane, RoutedCommand command, string header)
		{
			var mi = new MenuItem();
			mi.Command = command;
			mi.CommandParameter = pane;
			mi.CommandTarget = pane;
			mi.Header = header;
			mi.SetValue(DefaultStyleKeyProperty, XamDockManager.MenuItemStyleKey);
			return mi;
		}
		#endregion //CreateMenuItem

		#region CreateMenuItemSeparator
		private static Separator CreateMenuItemSeparator()
		{
			Separator sep = new Separator();
			sep.SetValue(DefaultStyleKeyProperty, XamDockManager.MenuItemSeparatorStyleKey);
			sep.SetResourceReference(FrameworkElement.StyleProperty, XamDockManager.MenuItemSeparatorStyleKey);
			return sep;
		}
		#endregion //CreateMenuItemSeparator

		#region OnExecuteDockFloatCommand
		private static void OnExecuteDockFloatCommand(object sender, ExecutedRoutedEventArgs e)
		{
			ContentPane cp = sender as ContentPane;

			if (cp == null)
				return;

			switch (cp.PaneLocation)
			{
				case PaneLocation.Unpinned:
					ContentPaneCommands.TogglePinnedState.Execute(cp, cp);

					// if it wasn't unpinned then exit
					if (cp.IsPinned == false)
						return;

					e.Handled = true;

					if (e.Command == FloatCommand)
					{
						// we have to asynchronously do the float part since it will be pinned asynchronously
						SendOrPostCallback floatPaneCallback = delegate(object param)
						{
							ContentPaneCommands.ToggleDockedState.Execute(param, param as ContentPane);
						};
						cp.Dispatcher.BeginInvoke(DispatcherPriority.Loaded, floatPaneCallback, cp);
					}
					break;
				case PaneLocation.DockedBottom:
				case PaneLocation.DockedLeft:
				case PaneLocation.DockedRight:
				case PaneLocation.DockedTop:
					if (e.Command == FloatCommand)
					{
						// switch to floating dockable states
						ContentPaneCommands.ToggleDockedState.Execute(cp, cp);
						e.Handled = true;
					}
					break;
				case PaneLocation.Floating:
					if (e.Command == DockCommand)
					{
						// switch to one of the dockable states
						ContentPaneCommands.ToggleDockedState.Execute(cp, cp);
						e.Handled = true;
					}
					break;
				case PaneLocation.Document:
				case PaneLocation.FloatingOnly:
					if (e.Command == FloatCommand)
					{
						ContentPaneCommands.ChangeToDockable.Execute(cp, cp);

						switch (cp.PaneLocation)
						{
							case PaneLocation.DockedBottom:
							case PaneLocation.DockedLeft:
							case PaneLocation.DockedRight:
							case PaneLocation.DockedTop:
								ContentPaneCommands.ToggleDockedState.Execute(cp, cp);
								break;
						}
					}
					else if (e.Command == DockCommand)
					{
						ContentPaneCommands.ChangeToDockable.Execute(cp, cp);

						if (cp.PaneLocation == PaneLocation.Floating)
							ContentPaneCommands.ToggleDockedState.Execute(cp, cp);
					}
					break;
			}
		}
		#endregion //OnExecuteDockFloatCommand

		#region OnCanExecuteDockFloatCommand
		private static void OnCanExecuteDockFloatCommand(object sender, CanExecuteRoutedEventArgs e)
		{
			ContentPane cp = sender as ContentPane;

			if (cp == null)
				return;

			var paneLocation = cp.PaneLocation;
			bool isUnpinned = paneLocation == PaneLocation.Unpinned;

			switch (paneLocation)
			{
				case PaneLocation.Unpinned:
				case PaneLocation.DockedBottom:
				case PaneLocation.DockedLeft:
				case PaneLocation.DockedRight:
				case PaneLocation.DockedTop:
					if (e.Command == FloatCommand)
					{
						e.CanExecute = cp.AllowDockingFloating ?? cp.AllowDocking;
						e.Handled = true;
					}
					else if (e.Command == DockCommand)
					{
						if (isUnpinned)
						{
							e.CanExecute = ContentPaneCommands.TogglePinnedState.CanExecute(cp, cp);
							e.Handled = true;
						}
						else
						{
							// already docked
							e.CanExecute = false;
							e.Handled = true;
						}
					}
					break;
				case PaneLocation.Floating:
					if (e.Command == DockCommand)
					{
						e.CanExecute = ContentPaneCommands.ToggleDockedState.CanExecute(cp, cp);
						e.Handled = true;
					}
					else if (e.Command == FloatCommand)
					{
						// already floating
						e.CanExecute = false;
						e.Handled = true;
					}
					break;
				case PaneLocation.FloatingOnly:
				case PaneLocation.Document:
					if (e.Command == FloatCommand)
					{
						e.CanExecute = cp.AllowDockingFloating ?? cp.AllowDocking;
						e.Handled = true;
					}
					else if (e.Command == DockCommand)
					{
						bool defaultAllowDocking = cp.AllowDocking;
						e.CanExecute = (cp.AllowDockingBottom ?? defaultAllowDocking)
							|| (cp.AllowDockingLeft ?? defaultAllowDocking)
							|| (cp.AllowDockingRight ?? defaultAllowDocking)
							|| (cp.AllowDockingTop ?? defaultAllowDocking);
						e.Handled = true;
					}
					break;
				default:
					e.CanExecute = false;
					e.Handled = true;
					break;
			}
		}
		#endregion //OnCanExecuteDockFloatCommand

		#endregion //Methods
	}
}
